﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Collections.Specialized;

namespace Benday.Timesheet.Presentation
{
    public class TimesheetEditorViewModel : ViewModelBase
    {
        public TimesheetEditorViewModel()
        {
            m_weeks = new TimesheetEditorWeeksViewModelCollection();
            m_weeks.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(m_weeks_CollectionChanged);
        }

        private string m_FirstName;
        public string FirstName
        {
            get
            {
                return m_FirstName;
            }
            set
            {
                m_FirstName = value;
                PropertyChangedHandler("FirstName");
            }
        }


        private string m_LastName;
        public string LastName
        {
            get
            {
                return m_LastName;
            }
            set
            {
                m_LastName = value;
                PropertyChangedHandler("LastName");
            }
        }

        private DateTime m_WeekStart;
        public DateTime WeekStart
        {
            get
            {
                return m_WeekStart;
            }
            set
            {
                m_WeekStart = value;
                PropertyChangedHandler("WeekStart");
            }
        }

        private DateTime m_WeekEnd;
        public DateTime WeekEnd
        {
            get
            {
                return m_WeekEnd;
            }
            set
            {
                m_WeekEnd = value;
                PropertyChangedHandler("WeekEnd");
            }
        }

        private TimesheetEditorWeeksViewModelCollection m_weeks;
        public TimesheetEditorWeeksViewModelCollection Weeks
        {
            get
            {
                return m_weeks;
            }
            set
            {
                m_weeks = value;
                
                PropertyChangedHandler("Weeks");                
            }
        }

        void m_weeks_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (TimesheetEditorTaskWeekViewModel week in e.NewItems)
                {
                    week.PropertyChanged += new PropertyChangedEventHandler(week_PropertyChanged);	
                }
            }

            PropertyChangedHandler("Weeks");            
        }

        void week_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "TotalHours")
            {
                PropertyChangedHandler("MondayTotal");
                PropertyChangedHandler("TuesdayTotal");
                PropertyChangedHandler("WednesdayTotal");
                PropertyChangedHandler("ThursdayTotal");
                PropertyChangedHandler("FridayTotal");
                PropertyChangedHandler("SaturdayTotal");
                PropertyChangedHandler("SundayTotal");
                PropertyChangedHandler("GrandTotal");
            }
        }


        private float GetTotalForDay(int dayIndex)
        {
            float total = 0f;

            foreach (var week in Weeks)
            {
                total += week.Days[dayIndex].Value;    
            }

            return total;
        }

        public float MondayTotal
        {
            get
            {
                return GetTotalForDay(0);
            }
        }

        public float TuesdayTotal
        {
            get
            {
                return GetTotalForDay(1);
            }
        }

        public float WednesdayTotal
        {
            get
            {
                return GetTotalForDay(2);
            }
        }

        public float ThursdayTotal
        {
            get
            {
                return GetTotalForDay(3);
            }
        }

        public float FridayTotal
        {
            get
            {
                return GetTotalForDay(4);
            }
        }

        public float SaturdayTotal
        {
            get
            {
                return GetTotalForDay(5);
            }
        }

        public float SundayTotal
        {
            get
            {
                return GetTotalForDay(6);
            }
        }

        private float GetGrandTotal()
        {
            float total = 0f;

            foreach (var week in Weeks)
            {
                foreach (var day in week.Days)
                {
                    total += day.Value;    
                }                
            }

            return total;   
        }

        public float GrandTotal
        {
            get
            {
                return GetGrandTotal();
            }
        }
    }
}