﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Benday.Timesheet.Presentation;
using System.Windows.Controls.Theming;
using System.Text;

namespace Benday.Timesheet.SilverlightUI
{
	public partial class TimesheetUserControl : UserControl
	{	
		private System.Windows.Threading.DispatcherTimer m_timer = new System.Windows.Threading.DispatcherTimer();

        public TimesheetUserControl() : this(true)
        {
            
        }

        public TimesheetUserControl(bool withDelayedDataContextPopulate)
        {
            InitializeComponent();
            this.LayoutUpdated += OnLayoutUpdated;

            if (withDelayedDataContextPopulate == true)
            {
                RequestDelayedPopulateDataContext();
            }
            else
            {
                PopulateDataContext();
            }
        }

        /// <summary>
        /// This gets around a bug in DataGrid.  If DataGrid has rows when a theme is applied, 
        /// then the existing row objects are marked as invisible.  After the rows are marked as 
        /// invisible, you'll always have blank rows in your DataGrid no matter what datasource you set.
        /// 
        /// This method sets the data context 1 second after the form is started.
        /// </summary>
        private void RequestDelayedPopulateDataContext()
        {
            m_timer.Interval = new TimeSpan(0, 0, 0, 1);
            m_timer.Tick += new EventHandler(m_timer_Tick);
            m_timer.Start();
        }

        void m_timer_Tick(object sender, EventArgs e)
        {
            m_timer.Stop();

            PopulateDataContext();
        }

        private void PopulateDataContext()
        {
            TimesheetEditorViewModel model = new TimesheetEditorViewModel();
            PopulateWithTestData(model);
            this.DataContext = model;
        }
        private void OnLayoutUpdated(object sender, EventArgs e)
        {
            LayoutUpdated -= OnLayoutUpdated;
            ImplicitStyleManager.SetApplyMode(LayoutRoot, ImplicitStylesApplyMode.Auto);
            ImplicitStyleManager.Apply(LayoutRoot); // redundant call to workaround datagrid style issue)
        } 
		
        private static void PopulateWithTestData(TimesheetEditorViewModel model)
        {
            model.FirstName = "Ben";
            model.LastName = "Day";
            model.WeekStart = new DateTime(2009, 9, 14);
            model.WeekEnd = new DateTime(2009, 9, 20);

            AddTestWeek(model, "Project 1");
            AddTestWeek(model, "Project 2");
            AddTestWeek(model, "Project 3");
        }

        private static void AddTestWeek(TimesheetEditorViewModel timesheet, string projectName)
        {
            TimesheetEditorTaskWeekViewModel week = new TimesheetEditorTaskWeekViewModel();

            week.Task.Value = projectName;

            timesheet.Weeks.Add(week);
        }

        private void m_btnAddRow_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            TimesheetEditorViewModel model = this.DataContext as TimesheetEditorViewModel;

            if (model != null)
            {
                AddTestWeek(model, Guid.NewGuid().ToString());
            }
        }

        private void m_btnSave_Click(object sender, System.Windows.RoutedEventArgs e)
        {
        	TimesheetEditorViewModel model = this.DataContext as TimesheetEditorViewModel;

            if (model != null)
            {
                StringBuilder output = new StringBuilder();

                output.AppendFormat("FirstName: {0}\r\n", model.FirstName);
                output.AppendFormat("LastName: {0}\r\n", model.LastName);
                output.AppendFormat("WeekStart: {0}\r\n", model.WeekStart);
                output.AppendFormat("WeekEnd: {0}\r\n", model.WeekEnd);

                output.AppendFormat("Task Week Count: {0}\r\n", model.Weeks.Count);

                foreach (var week in model.Weeks)
                {
                    output.AppendFormat("Task: {0}\r\n", week.Task.Value);
                    output.AppendFormat("   Monday:     {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Tuesday:    {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Wednesday:  {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Thursday:   {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Friday:     {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Saturday:   {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   Sunday:     {0}\r\n", week.Monday.Value);
                    output.AppendFormat("   TotalHours: {0}\r\n", week.TotalHours);
                }

                MessageBox.Show(output.ToString());
            }
        }

        private void m_btnTweekValues_Click(object sender, RoutedEventArgs e)
        {
            TimesheetEditorViewModel model = this.DataContext as TimesheetEditorViewModel;

            if (model != null)
            {
                model.FirstName = model.FirstName + "_";
                model.LastName = model.LastName + "_";
                model.WeekEnd = model.WeekEnd.AddDays(1);
                model.WeekStart = model.WeekStart.AddDays(1);

                foreach (var week in model.Weeks)
                {
                    foreach (var day in week.Days)
                    {
                        day.Value += 1;
                    }
                }
            }
        }

        private void m_btnBind_Click(object sender, RoutedEventArgs e)
        {
            PopulateDataContext();
        }        

	}
}